; ===========================================================================
; $Id: ucos51a.asm,v 1.31 1998/01/16 08:20:54 mandelli Exp $
;
; Description:
;	uC/OS-51 - Assembler functions for 8051
;
; Author[s]:
;	Gianpaolo Macario	<gianpi@geocities.com>
;
; Language:
;	BSO-Tasking ASM51 v4.0r0
; ===========================================================================

$CASE

$INCLUDE(..\MODEL)
$INCLUDE(..\MONITOR)

	NAME    UCOS51A

	; Make LINK51 complain if memory models are mixed up
	EXTRN	CODE(%MODEL)	

	; WARNING: the following macros, extern and global var definitions
	; must match with ucos51c.h, ucos.h:
	;
	%*define (OS_SYS_TIMER)(0)	; 0=Timer0, 1=Timer1
	;
	extrn   data(_OSIntNesting)	; UBYTE    _data OSIntNesting;
	extrn   data(_OSTCBCur)		; OS_TCB * _data OSTCBCur;
	extrn   data(_OSTCBHighRdy)	; OS_TCB * _data OSTCBHighRdy;
	extrn   data(_OSTimReload)	; UWORD    _data OSTimReload;
	;
	;extrn	code(_?OSIntEnter)	; void OSIntEnter(void);
	extrn	code(_?OSIntExit)	; void OSIntExit(void);

	extrn	data(__SP)		; CC51 Virtual Stack Pointer

?STACK  SEGMENT IDATA
        RSEG    ?STACK
OSStack:
        ds  	40h
OSStkStart	idata OSStack-1       	; TBD: Should use __STKSTART
        ;;extrn   idat(__STKSTART)

; -------------------------------------------------------------------------
; OSFRAMESIZE is the number of bytes saved to stack
; by a SAVE_ALL macro (+2 for PCL, PCH saved with a LCALL)
;
; WARNING: This should match with SAVE_ALL, REST_ALL macros
;	   and OSTaskCreate() in ucos51c.c
;
; Both the TCB and the system stack contain
; the OS context in this format:
;
; high addr:	SP
;		__SP+1	; BSO-Tasking CC51
;		__SP	; Virtual Stack Pointer
;		R7
;		R6
;		R5
;		R4
;		R3
;		R2
;		R1
;		R0
;		DPH
;		DPL
;		B
;		ACC
;		PSW
;		PCH
;		PCL
;		user data bytes
; low addr:
;
; In TCB stack, SP contains the number of bytes
; saved after OS context (i.e. user data in stack)
;
OSFRAMESIZE	equ 18			; 16 + sizeof(__SP)

; -------------------------------------------------------------------------
; Macros
; -------------------------------------------------------------------------

%*DEFINE (SAVE_ALL)
(
	; Save current task context
	;
	; PCL, PCH have already been pushed by lcall/interrupt
	;
	push	psw		
	push	acc
	push	b
	push	dpl
	push	dph
	;
	push	ar0
	push	ar1
	push	ar2
	push	ar3
	push	ar4
	push	ar5
	push	ar6
	push	ar7
	;
	; --------------------------------
	; Save CC51 Virtual Stack Pointer
	;
	push	__SP
	push	__SP+1
	; --------------------------------
	;
	push	sp
)

%*DEFINE (REST_ALL)
(
	; Restore task context
	;
	pop 	acc		; SP: discard it
	;
	; -----------------------------------
	; Restore CC51 Virtual Stack Pointer
	;
	pop	__SP+1
	pop	__SP
	; -----------------------------------
	;
	pop	ar7
	pop	ar6
	pop	ar5
	pop	ar4
	pop	ar3
	pop	ar2
	pop	ar1
	pop	ar0
	;
	pop	dph
	pop	dpl
	pop 	b
	pop 	acc
	pop 	psw
	;
	; ret/reti will pop PCH, PCL
)

%*DEFINE (LCALL_VEC(vecnum)) LOCAL here
(
	; Simulate lcall @revectab[%vecnum]
	;
	mov	dptr, #revectab + (2 * %vecnum)
	movx	a, @dptr
	xch	a, b
	inc	dptr
	movx	a, @dptr
	mov	dph, b
	mov	dpl, a
	mov	a, #low %here
	push	acc
	mov	a, #high %here
	push	acc
	clr	a
	jmp	@a+dptr
%here:
)

; Determine Task stack start and size
;
; Parameters:
;	DPTR:	Current Task StkPtr
;
; Returns:
;	DPTR:	Beginning of Task stack -1
;	R5:	Number of bytes in Task stack
;
; Modifies:
;	ACC R0 R1
;
%*DEFINE (GETSTKINFO)
(
	; Determine Task stack start and size
	;
	movx	a, @dptr		; stk = OSTCBCur->StkPtr
	mov	r0, a
	inc	dptr
	movx	a, @dptr
	mov	r1, a			; R01 = stk

	mov	dph, r0
	mov	dpl, r1
	movx	a, @dptr		; Get User Stack Ptr (*stk)

	add 	a, #OSFRAMESIZE
	mov 	r5, a			; R5 = #bytes to copy

	mov 	a, dpl
	clr	c
	subb 	a, r5
	mov 	dpl, a
	mov 	a, dph
	subb 	a, #0
	mov 	dph, a			; DPTR = srcptr - 1
)

; Copy system stack to TCB stack
;
; Parameters:
;	DPTR:	beginning of TCB stack -1
;	R1:	beginning os system stack -1
;	R5:	number of bytes to copy
;
; Returns:
;	DPTR:	set at top of the TCB stack
;	R1:	set at top of system stack
;
; Modifies:
;	ACC DPTR R4
;
%*DEFINE (CPSTK_S2T) LOCAL savestk2
(
	; Copy system stack to TCB stack
	;
	mov	r4, ar5			; R4 = count
%savestk2:
	inc	r1
	inc	dptr
	mov	a, @r1
	movx	@dptr, a
	djnz	R4, %savestk2
)

; Copy TCB stack to system stack, then adjust SP accordingly
;
; Parameters:
;	DPTR:	beginning of TCB stack -1
;	R1:	beginning os system stack -1
;	R5:	number of bytes to copy
;
; Returns:
;	DPTR:	set at top of the TCB stack
;	R1:	set at top of the new system stack
;	SP:	set at top of the new system stack
;
; Modifies:
;	ACC R4
;
%*DEFINE (CPSTK_T2S) LOCAL copystk1
(
	; Copy TCB stack to system stack, then adjust SP accordingly
	;
	mov 	r4, ar5			; R4 = count
%copystk1:
	inc 	dptr
	inc 	R1
	movx 	a, @dptr
	mov 	@R1, a
	djnz 	R4, %copystk1
	; Now R1 is the new task SP
	mov 	sp, r1
)

; Fixup SP value in TCB stack
;
; Parameters:
;	DPTR:	pointer to new top of TCB stack
;	R5:	copied stack size (including OSFRAMESIZE)
;
; Modifies:
;	ACC
;
%*DEFINE (FIX_TSTK_SP)
(
	; Fixup SP value in TCB stack
	;
	mov	a, r5			; R5 hold stack size!
	clr	c
	subb	a, #OSFRAMESIZE
	movx	@dptr, a		; Fix SP in user stack
)

; Fixup field OSTCBCur->OSTCBStkPtr
;
; Parameters:
;	DPTR:	pointer to new top of TCB stack
;
; Modifies:
;	ACC R0 R1 DPTR
;
%*DEFINE (FIX_CUR_STKPTR)
(
	; Fixup field OSTCBCur->OSTCBStkPtr
	;
	mov	r0, dph			; R01 = stk
	mov	r1, dpl
	;
	%LDPTR(_OSTCBCur)
	;
	mov	a, r0
	movx	@dptr, a
	inc	dptr
	mov	a, r1
	movx	@dptr, a		; OSTCBCur->OSTCBStkPtr = stk
)

; Reload Timer register when timer overflowed.
; Parameters:
;	num		(0=timer0, 1=timer1)
;
%*DEFINE (RELOAD_TIMER(num))
(
	; Reload Timer%num register
	;
	clr	tr%num			; Stop Timer%num
	;
	mov	a,_OSTimReload+1	; [~1] (TH%num,TL%num) += OSTimReload
	add	a,tl%num		; [~1]
	mov	tl%num,a		; [~1]
	mov	a,_OSTimReload		; [~1]
	addc	a,th%num		; [~1]
	mov	th%num,a
	;
	mov	a,#14			; [~1] Take into account time elapsing
	add	a,tl%num		; [~1] from "clr  tr%num"
	mov	tl%num,a		; [~1] to   "setb tr%num"
	clr	a			; [~1]
	addc	a,th%num		; [~1]
	mov	th%num,a		; [~1]
	;
	setb	tr%num			; [~1] Restart Timer%num
)

; Skeleton of a ucos51-aware Interrupt Service Routine
;
%*DEFINE (UCOS51_ISR(vecnum, label))
(
	public	%label
%label:
	%SAVE_ALL
	inc	_OSIntNesting		; Faster than calling OSIntEnter()
	;lcall	_?OSIntEnter		; ...the old way...
	;
	; DBG: (vecnum == %vecnum), (OS_SYS_TIMER == %OS_SYS_TIMER)
	;
	; if (vecnum == 1 && OS_SYS_TIMER == 0) RELOAD_TIMER(0)
	%IF (%EQS(%vecnum,1)) THEN (%IF (%EQS(%OS_SYS_TIMER,0)) THEN (
	%RELOAD_TIMER(0)
	) FI) FI
	; if (vecnum == 3 && OS_SYS_TIMER == 1) RELOAD_TIMER(1)
	%IF (%EQS(%vecnum,3)) THEN (%IF (%EQS(%OS_SYS_TIMER,1)) THEN (
	%RELOAD_TIMER(1)
	) FI) FI
	;
	%LCALL_VEC(%vecnum)
	lcall	_?OSIntExit
	ljmp	popall3	
)

; Load a 16-bit variable into DPTR
;
%*DEFINE (LDPTR(value))
(
	mov	dph, %value+0		; DPTR = %value
	mov	dpl, %value+1
)

; Save DPTR into a 16-bit variable
;
%*DEFINE (SDPTR(value))
(
	mov	%value+0, dph		; %value = DPTR
	mov	%value+1, dpl
)

; -------------------------------------------------------------------------
; Start Multitasking
;
; 	void OSStartHighRdy(void)

UCOS51A_OSSTARTHIGHRDY_PR   SEGMENT CODE
	RSEG    UCOS51A_OSSTARTHIGHRDY_PR
	public  _?OSStartHighRdy

_?OSStartHighRdy:

	; OSTCBCur = OSTCBHighRdy
	%LDPTR(_OSTCBHighRdy)
	%SDPTR(_OSTCBCur)

	; Determine HighRdy Task stack start and size
	%GETSTKINFO

	mov     r1, #OSStkStart       	; R1 = destptr - 1

	; Copy HighRdy Task stack to system stack and adjust SP
	%CPSTK_T2S

	; Pop all user register from task stack
popall:
	%REST_ALL
	setb	ea			; Enable interrupts again
	ret				; Pop PCH,PCL (DBG: was reti!)

; -------------------------------------------------------------------------
; Perform a context switch (from task level)
;
;	void OSCtxSw(void)

UCOS51A_OSCTXSW_PR   SEGMENT CODE
	RSEG    UCOS51A_OSCTXSW_PR
	public  _?OSCtxSw

_?OSCtxSw:
	%SAVE_ALL

	; Save system stack in OSTCBCur TCB stack
	%LDPTR(_OSTCBCur)

	; Determine beginning of OSTCBCur TCB stack
	%GETSTKINFO

	; DPTR now points to the beginning of the user stack-1

	mov 	R1, #OSStkStart		; R1 = #OSStack - 1
	mov	a, sp
	clr	c
    	subb    a, r1
	mov	r5, a			; R5 = #bytes to copy

	; Copy system stack to TCB stack
	%CPSTK_S2T
	
	; Fixup SP value in TCB stack
	%FIX_TSTK_SP

	; Fixup OSTCBCur->OSTCBStkPtr
	%FIX_CUR_STKPTR
	
	; Point to HI Prio. Task Rdy
	;
	ljmp    _?OSStartHighRdy

; -------------------------------------------------------------------------
; Perform a context switch (from an ISR)
;
; We assume that a SAVE_ALL macro
; has already been made by UCOS51_ISR.
;
;	void OSIntCtxSw(void)

UCOS51A_OSINTCTXSW_PR   SEGMENT CODE
	RSEG    UCOS51A_OSINTCTXSW_PR
	public  _?OSIntCtxSw

_?OSIntCtxSw:
	; Ignore data in stack:
	;   - call to OSIntExit  (2 bytes)
	;   - call to OSIntCtxSw (2 bytes)
	;   - locals		 (0 bytes)
	;
	mov	a, sp
	clr	c
	subb	a, #4
	mov	sp, a

	; From now on there is a little bit of OSCtxSw:

	; Save system stack in OSTCBCur TCB stack
	%LDPTR(_OSTCBCur)

	; Determine beginning of OSTCBCur TCB stack
	%GETSTKINFO

	; DPTR now points to the beginning of the user stack-1

	mov 	R1, #OSStkStart		; R1 = #OSStack - 1
	mov	a, sp
	clr	c
    	subb    a, r1
	mov	r5, a			; R5 = #bytes to copy

	; Copy system stack to TCB stack
	%CPSTK_S2T
	
	; Fixup SP value in TCB stack
	%FIX_TSTK_SP

	; Fixup OSTCBCur->OSTCBStkPtr
	%FIX_CUR_STKPTR

	; From now on there is a little bit of OSStartHighRdy:

	; OSTCBCur = OSTCBHighRdy
	%LDPTR(_OSTCBHighRdy)
	%SDPTR(_OSTCBCur)

	; Determine HighRdy Task stack start and size
	%GETSTKINFO

	mov     r1, #OSStkStart       	; R1 = destptr - 1

	; Copy HighRdy Task stack to system stack and adjust SP
	%CPSTK_T2S

	; Pop all user register from task stack
popall3:
	%REST_ALL
	setb	ea			; Enable interrupts again (?)
	reti				; Pop PCH,PCL

; -------------------------------------------------------------------------
; Revector Hardware Interrupts
;
%IF( %EQS(%PAULMON,RAM) ) THEN (
INTVECT_LOC	EQU	8000h		; PAULMON-RAM
) ELSE (
	%IF( %EQS(%PAULMON,ROM)) THEN (
INTVECT_LOC	EQU	2000h		; PAULMON-ROM
	) ELSE (
INTVECT_LOC	EQU	0000h		; Application without monitor
	) FI
) FI

	CSEG	AT INTVECT_LOC+0003h	; IE0
	ljmp	__CIH00

	CSEG	AT INTVECT_LOC+000Bh	; TF0
	ljmp	__CIH01

	CSEG	AT INTVECT_LOC+0013h	; IE1
%IF( %EQS(%PAULMON,ROM)) THEN (
	ljmp	14EFh			; PAULMON Single-Step ISR
) ELSE (
	ljmp	__CIH02
) FI

	CSEG	AT INTVECT_LOC+001Bh	; TF1
	ljmp	__CIH03

	CSEG	AT INTVECT_LOC+0023h	; RI_TI (SIO0)
	ljmp	__CIH04

	CSEG	AT INTVECT_LOC+002Bh	; SIO1 (8052 and derivatives)
	ljmp	__CIH05

	; CSEG	AT INTVECT_LOC+0033h	; UNK6 (not handled by PAULMON)
	; ljmp	__CIH06

	; CSEG	AT INTVECT_LOC+003Bh	; UNK7 (not handled by PAULMON)
	; ljmp	__CIH07

; -------------------------------------------------------------------------
; Interrupt Service Routines

UCOS51A_ISR_PR	SEGMENT CODE
	RSEG	UCOS51A_ISR_PR

	; Interrupt 0 (INT_IE0) Service Routine
	%UCOS51_ISR(0, __CIH00)

	; Interrupt 1 (INT_TF0) Service Routine
	%UCOS51_ISR(1, __CIH01)

	; Interrupt 2 (INT_IE1) Service Routine
	%UCOS51_ISR(2, __CIH02)

	; Interrupt 3 (INT_TF1) Service Routine
	%UCOS51_ISR(3, __CIH03)

	; Interrupt 4 (INT_RI_TI) Service Routine
	%UCOS51_ISR(4, __CIH04)

	; Interrupt 5 (INT_UNK5) Service Routine
	%UCOS51_ISR(5, __CIH05)

	; Interrupt 6 (INT_UNK6) Service Routine
	%UCOS51_ISR(6, __CIH06)

	; Interrupt 7 (INT_UNK7) Service Routine
	%UCOS51_ISR(7, __CIH07)

; -------------------------------------------------------------------------
; Table of vectors in RAM
;
UCOS51A_XD   SEGMENT XDATA
	RSEG    UCOS51A_XD

revectab:
	ds	2	; vector of CIH00
	ds	2	; vector of CIH01
	ds	2	; vector of CIH02
	ds	2	; vector of CIH03
	ds	2	; vector of CIH04
	ds	2	; vector of CIH05
	ds	2	; vector of CIH06
	ds	2	; vector of CIH07

; -------------------------------------------------------------------------
; Return current interrupt vector
;
; C prototype:
;	FUNC_PTR getvect(u_int8_t vecnum)
;
UCOS51A_GETVECT_PR   SEGMENT CODE
	RSEG    UCOS51A_GETVECT_PR
	public	_?getvect

_?getvect:
	mov	a, r7		; get vecnum
	anl	a, #07h		; just make sure
	rl	a		; multiply by 2

	mov	dph, #high revectab
	mov	dpl, #low  revectab

	add	a, dpl
	mov	dpl, a
	mov	a, dph
	addc	a, #0
	mov	dph, a		; dptr = #revectab + (2 * vecnum)

	movx	a, @dptr
	mov	r6, a		; High(result)
	inc	dptr
	movx	a, @dptr
	mov	r7, a		; Low(result)
	ret

; -------------------------------------------------------------------------
; Change interrupt vector
;
; C prototype:
;	void setvect(u_int8_t vecnum, FUNC_PTR newfunc)
;
UCOS51A_SETVECT_PR   SEGMENT CODE
	RSEG    UCOS51A_SETVECT_PR
	public	_?setvect

_?setvect:
	mov	a, r7		; get vecnum
	anl	a, #07h		; just make sure
	rl	a		; multiply by 2

	mov	dph, #high revectab
	mov	dpl, #low  revectab

	add	a, dpl
	mov	dpl, a
	mov	a, dph
	addc	a, #0
	mov	dph, a		; dptr = #revectab + (2 * vecnum)

	mov	a, r4		; High(newfunc)
	movx	@dptr, a
	inc	dptr
	mov	a, r5		; Low(newfunc)
	movx	@dptr, a
	ret

; -------------------------------------------------------------------------

; end of program

	end
