/* $Id: ucos51c.c,v 1.18 1998/01/16 08:20:54 mandelli Exp $
 *
 * Description:
 *	uC/OS-51 - Processor Dependent Module
 *
 * Author[s]:
 *	Jean Labrosse
 *	Gianpaolo Macario	gianpi@geocities.com
 *
 * Conditionalizations:
 *	_CC51			BSO-Tasking CC51 v4.0r0
 */

/*
***********************************************************
*                     INCLUDE FILES
***********************************************************
*/
#include "includes.h"

#if !defined(_CC51)
#error This piece of code only works with BSO-Tasking CC51 V4.0r0
#endif

/*
***********************************************************
*       		GLOBAL VARIABLES
***********************************************************
*/
UWORD  _data OSTimReload = 0;	/* 0=maximum time between ticks */


/*
***********************************************************
*	CONVERT MICROSECONDS TO AND FROM SYSTICKS
***********************************************************
*/
UWORD OS_usec2cnt(unsigned long t_us)
{
  unsigned long temp;

  temp  = t_us;
  temp *= (OS_CPU_CLOCK / 12L) / 20000;
  temp /= 1000000L / 20000;
  temp  =  ~temp;
  return temp & 0x0000FFFFL;
}	/* OS_usec2cnt() */

unsigned long OS_cnt2usec(UWORD cnt)
{
  unsigned long temp;

  temp  = ~cnt & 0x0000FFFFL;
  temp *= 1000000L / 20000;		/* never overflows */
  temp /= (OS_CPU_CLOCK / 12L) / 20000;
  return temp;
}	/* OS_cnt2usec() */

/*
***********************************************************
*                    CHANGE SYSTEM TIMER FREQUENCY
***********************************************************
*/
void OSSetTimReload(UWORD count)
{
  OS_ENTER_CRITICAL();
  OSTimReload = count;
  OS_EXIT_CRITICAL();
}	/* OSSetTimReload() */

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/
UBYTE OSTaskCreate(
	void  _regparm _reentrant (*taskp)(void *pd),
	void  *tskdata,
	void  *pstk,
        size_t stksize,
	UBYTE  p)
{
    OS_STK_TYPE *stk;
    OS_STK_TYPE *vstack_ptr;
    UBYTE err;

    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] != (OS_TCB *)0) {
	/* Avoid creating task if already exist */
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }

    stk		= (OS_STK_TYPE *)pstk - 1;
    vstack_ptr  = (OS_STK_TYPE *)pstk + stksize - 1;

    /* WARNING: This code must match with ucos51a.asm */

    *++stk      = ((UWORD)taskp & 0xFF);            	/* PCL    */
    *++stk      = ((UWORD)taskp >> 8);              	/* PCH    */
    *++stk      = 0x00;					/* PSW    */
    *++stk      = 0x00;					/* ACC    */
    *++stk      = 0x00;					/* B      */
    *++stk      = 0x00;					/* DPL    */
    *++stk      = 0x00;					/* DPH    */
    *++stk      = 0x00;					/* R0     */
    *++stk      = 0x00;					/* R1     */
    *++stk      = 0x00;					/* R2     */
    *++stk      = 0x00;					/* R3     */
    *++stk      = 0x00;					/* R4     */
    *++stk      = 0x00;					/* R5     */
    *++stk      = ((UWORD)tskdata >> 8);		/* R6     */
    *++stk      = ((UWORD)tskdata & 0xFF);		/* R7     */
    *++stk	= ((UWORD)vstack_ptr >> 8);		/* __SP   */
    *++stk	= ((UWORD)vstack_ptr & 0xFF);		/* __SP+1 */
    *++stk      = 0x00;					/* SP     */

    err = OSTCBInit(p, (void *)stk);    /* Get and initialize a TCB */

    if (err == OS_NO_ERR) {
        if (OSRunning) {
            /* Find highest priority task if multitasking has started */
            OSSched();
        }
    }
    return (err);
}	/* OSTaskCreate() */


/* === End of File === */
