/* $Id: ucos51c.h,v 1.18 1998/01/16 08:20:55 mandelli Exp $
 *
 * Description
 *      uC/OS-51 - Processor dependent module interface
 *
 * Author[s]:
 *      Jean Labrosse
 *      Gianpaolo Macario       gianpi@geocities.com
 *
 * Conditionalizations:
 *      _CC51                   BSO-Tasking CC51 v4.0r0
 *      __C51__                 Franklin C51 v6.11
 *      __C251__                Keil C251 v1.1
 */

#ifndef _UCOS51C_H_
#define _UCOS51C_H_

#if defined(_CC51)
/* BSO-Tasking CC51 v4.0r0 */
#elif defined(__C51__)
/* Franklin C51 v6.11 */
/* TBD */
#elif defined(__C251__)
/* Keil C251 v1.1 */
#define _data           data
#define _large          /* ??? */
#define _rom            /* ??? */
#define _reentrant      /* ??? */
#define _regparm        /* ??? */
#define _small          /* ??? */
#define _xdat           xdata
#else
#error Compiler/target not supported so far
#endif

/*
************************************************************
*                      CONSTANTS
************************************************************
*/
#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif


/*
************************************************************
*                        MACROS
************************************************************
*/
#define  OS_ENTER_CRITICAL()    (EA = 0)
#define  OS_EXIT_CRITICAL()     (EA = 1)
#define  OS_TASK_SW()           OSCtxSw()

/* The following macros must match with ucos51a.asm */
#define  OS_SYS_TIMER   0       /* 0=Timer0, 1=Timer1 */

/*
************************************************************
*                       DATA TYPES
************************************************************
*/
typedef char            BOOLEAN;    /* Cannot use _bit in structures */
typedef unsigned char   UBYTE;
typedef signed   char   BYTE;
typedef unsigned int    UWORD;
typedef signed   int    WORD;
typedef unsigned long   ULONG;
typedef signed   long   LONG;

typedef unsigned char   u_char;

#define int8_t          BYTE
#define int16_t         WORD
#define int32_t         LONG

#define u_int8_t        UBYTE
#define u_int16_t       UWORD
#define u_int32_t       ULONG

/*
************************************************************
*                       GLOBAL VARIABLES
************************************************************
*/

/* THx,TLx reload values */
extern UWORD _data OSTimReload;         

/*
************************************************************
*                       FUNCTION PROTOTYPES
************************************************************
*/

/* Prototypes of functions defined in ucos51a.asm: */

/* Start Multitasking */
void OSStartHighRdy(void);

/* Perform a context switch (from task level) */
void OSCtxSw(void);

/* Perform a context switch (from an ISR) */
void OSIntCtxSw(void);

/* Get System Timer frequency */
UWORD OSGetTimReload(void);

/* Change System Timer frequency (0=lowest, 0xFFFF=highest) */
void OSSetTimReload(UWORD count);

/* Convert systick in microseconds to a count for OSSetTimReload() */
UWORD OS_usec2cnt(unsigned long t_us);

/* Like OS_usec2cnt(), but only for a constant t_us
 * (example: #define t_us 10000L) 
 */
UWORD OS_USEC2CNT(unsigned long t_us);

/* Convert count to microseconds (inverse of OS_usec2cnt()) */
unsigned long OS_cnt2usec(UWORD cnt);

/* Get system tick time (in microseconds) */
unsigned long OSGetSystick(void);

/* Set system tick time (in microseconds) */
void OSSetSystick(unsigned long t_us);


/* Prototypes of getvect(), setvect() are in isr51.h */

/* Prototypes of functions defined in ucos51c.c: */
/* Already prototyped in ucos.h */

/* Inline function implementation */
#define OSGetTimReload()        (OSTimReload)
#define OS_USEC2CNT(t_us)  \
	(~((UWORD)((OS_CPU_CLOCK / 12)*(t_us)) / 1000000L) & 0xFFFF)
#define OSGetSystick()          OS_cnt2usec(OSGetTimReload())
#define OSSetSystick(t_us)      OSSetTimReload(OS_usec2cnt(t_us))

/*
************************************************************
*                       MISC
************************************************************
*/
#define OS_FAR          /* _large */    /* x86-only */
#define OS_NEAR                         /* x86-only */
#define OS_ROMDATA      _rom            /* const data in ROM */
#define OS_STK_TYPE     UBYTE           /* Stack is byte-organized */

#endif  /* ndef _UCOS51C_H_ */

/* === End of File === */
